



using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Threading.Tasks;
using Aurora.Common.Entities;
using Aurora.Core.Domain.Entities;
using Aurora.Infrastructure.Tenant;
using Aurora.Infrastructure.User;
using Microsoft.EntityFrameworkCore;

namespace Aurora.Core.EntityFramework
{
    public class ApplicationDbContext : ApplicationReadonlyDbContext
    {
        public ApplicationDbContext(DbContextOptions<ApplicationDbContext> options)
        : base(options)
        {
            // ,
            // ICurrentUserProvider currentUserProvider,
            // ICurrentTenantProvider currentTenantProvider
        }

        public override int SaveChanges()
        {
            UpdateCommonFileds();
            return base.SaveChanges();
        }

        public override async Task<int> SaveChangesAsync(CancellationToken cancellationToken = default(CancellationToken))
        {
            UpdateCommonFileds();
            return await base.SaveChangesAsync(cancellationToken);
        }

        private void UpdateCommonFileds()
        {
            var nowTime = DateTime.Now;

            if (string.IsNullOrEmpty(currentUser.UserName))
            {
                var tcode = currentTenant.TenantCode;

            }

            foreach (var entry in this.ChangeTracker.Entries<TenantEntityBase>().Where(x => x.State == EntityState.Added || x.State == EntityState.Modified))
            {
                var entity = entry.Entity;
                switch (entry.State)
                {
                    case EntityState.Added:
                        if (entity.TenantCode == "")
                            entity.TenantCode = currentTenant.TenantCode;
                        entity.CreateTime = nowTime;
                        entity.CreateUserName = currentUser.UserName;
                        break;
                    case EntityState.Modified:
                        entity.ModifyTime = nowTime;
                        entity.ModifyUserName = currentUser.UserName;
                        break;
                }
            }
            this.ChangeTracker.DetectChanges();
        }
    }
}